﻿using System;
using System.Collections.Generic;
using System.Text;
using Library;
using Kuzushi.Resources;
using System.Drawing;
using Tsukikage.GameSDK.Direct3D;
using Tsukikage.GameSDK.Util;

namespace Kuzushi.Engine
{
    /// <summary>
    /// ゲームエンジン
    /// </summary>
    public class Engine
    {
        /// <summary>
        /// ブロックのリスト
        /// </summary>
        List<GBlock> blocks;

        /// <summary>
        /// ボール
        /// </summary>
        public GBall Ball;

        /// <summary>
        /// ラケット
        /// </summary>
        public GRacket Racket;

        /// <summary>
        /// 現在のスコア
        /// </summary>
        int score = 0;

        /// <summary>
        /// ハイスコア
        /// </summary>
        internal int highScore = 0;

        /// <summary>
        /// 残りボール
        /// </summary>
        int lives = 5;

        /// <summary>
        /// ゲーム開始からの経過フレーム数
        /// </summary>
        int frameCounter = 0;

        /// <summary>
        /// コンストラクタ(初期化)
        /// </summary>
        public Engine()
        {
            this.blocks = new List<GBlock>();
            
            this.Ball = new GBall(this);
            this.Racket = new GRacket();
        }

        /// <summary>
        /// 内部処理
        /// 毎フレーム呼ばれる
        /// </summary>
        public void Internal()
        {
            // フレームカウンタを増やす
            frameCounter++;

            // オブジェクトの内部処理
            foreach (GBlock block in blocks)
                block.Internal();

            // ラケット内部処理
            Racket.Internal();

            // ライフがある
            if (lives > 0)
            {

                // 壁の処理
                if (Ball.PosX < 0 + Ball.Radius) Ball.SpdX = Math.Abs(Ball.SpdX);
                if (Ball.PosX > 640 - Ball.Radius) Ball.SpdX = -Math.Abs(Ball.SpdX);
                if (Ball.PosY < 0 + Ball.Radius) Ball.SpdY = Math.Abs(Ball.SpdY);

                // 下方向は
                if (Ball.PosY > 480 - Ball.Radius)
                {
                    // ライフを減らしてボールを初期化
                    lives--;
                    Ball = new GBall(this);
                    Sounds.Fall.PlayOnce();
                }

                // ボール内部処理
                Ball.Internal();

                // 生存点を加算
                score++;
            }

            // 一定時間ごとにランダムにブロックを追加
            if (frameCounter % 30 == 0)
            {
                XSRandom rnd = new XSRandom();
                blocks.Add(new GBlock(
                    new Argb(255, rnd.Next(2) * 64 + 127, rnd.Next(2) * 64 + 127, rnd.Next(2) * 64 + 127),
                    rnd.Next(640), 0, rnd.Next(50) + 30));
            }


            // スコアがハイスコアを超えていたら更新
            if (score > highScore)
                highScore = score;
        }

        /// <summary>
        /// 描画処理
        /// 毎フレーム呼ばれるとは限らない
        /// </summary>
        public void Draw()
        {
            // ブロックを描く
            foreach (GBlock block in blocks)
                block.Draw();

            // ラケットを描く
            Racket.Draw();

            // ライフがあるならボールを描く
            if (lives > 0)
                Ball.Draw();

            // スコア表示
            Fonts.Uzura18pt.DrawText(new Point(0, 0), "Score: " + score.ToString("######"), Argb.White);
            Fonts.Uzura18pt.DrawText(new Point(0, 24), "High : " + highScore.ToString("######"), Argb.White);
            Fonts.Uzura18pt.DrawText(new Point(0, 48), "Lives: " + new string('●', lives), Argb.White);
        }

        /// <summary>
        /// ブロックを取り除く
        /// </summary>
        /// <param name="block">取り除くブロック</param>
        public void RemoveBlock(GBlock block)
        {
            blocks.Remove(block);
        }

        /// <summary>
        /// Actionで指定した動作をすべてのブロックに適用します。
        /// C#のデリゲートという機能。覚えておくと便利……？
        /// </summary>
        /// <param name="action">動作</param>
        public void ForeachBlocks(Action<GBlock> action)
        {
            foreach (GBlock block in blocks.ToArray())
                action(block);
        }

        /// <summary>
        /// 点数を加算
        /// </summary>
        /// <param name="score"></param>
        public void AddScore(int score)
        {
            this.score += score;
        }
    }
}
